#ifndef DAC81416_H
#define DAC81416_H
#include <stdint.h>

typedef struct DAC81416_data{
    union {
        uint32_t data;
        uint8_t data_serial[4];
    };
}DAC81416_data_t;

#define NOP         0x00 //NOP Register
#define DEVICEID    0x01 //Device ID Register
#define STATUS      0x02 //Status Register
#define SPICONFIG   0x03 //SPI Configuration Register
#define GENCONFIG   0x04 //General Configuration Register
#define BRDCONFIG   0x05 //Broadcast Configuration Register
#define SYNCCONFIG  0x06 //Sync Configuration Register
#define TOGGCONFIG0 0x07 //DAC[15:8] Toggle Configuration Register
#define TOGGCONFIG1 0x08 //DAC[7:0] Toggle Configuration Register
#define DACPWDWN    0x09 //DAC Power-Down Register
#define DACRANGE0   0x0A //DAC[15:12] Range Register
#define DACRANGE1   0x0B //DAC[11:8] Range Register
#define DACRANGE2   0x0C //DAC[7:4] Range Register
#define DACRANGE3   0x0D //DAC[3:0] Range Register
#define TRIGGER     0x0E //Trigger Register
#define BRDCAST     0x0F //Broadcast Data Register
#define DAC0        0x10 //DAC0 Data Register
#define DAC1        0x11 //DAC1 Data Register
#define DAC2        0x12 //DAC2 Data Register
#define DAC3        0x13 //DAC3 Data Register
#define DAC4        0x14 //DAC4 Data Register
#define DAC5        0x15 //DAC5 Data Register
#define DAC6        0x16 //DAC6 Data Register
#define DAC7        0x17 //DAC7 Data Register
#define DAC8        0x18 //DAC8 Data Register
#define DAC9        0x19 //DAC9 Data Register
#define DAC10       0x1A //DAC10 Data Register
#define DAC11       0x1B //DAC11 Data Register
#define DAC12       0x1C //DAC12 Data Register
#define DAC13       0x1D //DAC13 Data Register
#define DAC14       0x1E //DAC14 Data Register
#define DAC15       0x1F //DAC15 Data Register
#define OFFSET0     0x20 //DAC[14-15;12-13] Differential Offset Register
#define OFFSET1     0x21 //DAC[10-11;8-9] Differential Offset Register
#define OFFSET2     0x22 //DAC[6-7;4-5] Differential Offset Register
#define OFFSET3     0x23 //DAC[2-3;0-1] Differential Offset Register

#define READ_BIT 0x80
void DAC81416_init(void);
void DAC81416_write_register(uint8_t reg_to_write, uint16_t config);
uint16_t DAC81416_read_register(uint8_t reg_to_read);

//read out registers
#define DEVICE_ID_VAL 0x29C0
#define VERSION_ID_VAL 0x0000
#define DEVICE_ID_MSK 0xFFF0
#define VERSION_ID_MSK 0x000F
uint16_t DAC81416_get_device_id(void);

#define CRC_ALM_MSK  0x02
#define DAC_BUSY_MSK 0x01
#define TEMP_ALM_MSK 0x00
uint16_t DAC81416_get_status(void);

//SPI CONFIG params
#define TEMP_ALM_EN    (1 << 11)
#define DAC_BUSY_EN    (1 << 10)
#define CRC_ALM_EN     (1 << 9)
#define SOFT_TOGGLE_EN (1 << 6)
#define DEV_PW_DWN     (1 << 6)
#define CRC_EN         (1 << 4)
#define STR_EN         (1 << 3)
#define SDO_EN         (1 << 2)
#define FSDO           (1 << 1)

//GEN CONFIG params
#define REF_PWDWN        (1 << 14)
#define DAC_1415_DIFF_EN (1 << 7)
#define DAC_1213_DIFF_EN (1 << 6)
#define DAC_1011_DIFF_EN (1 << 5)
#define DAC_89_DIFF_EN   (1 << 4)
#define DAC_67_DIFF_EN   (1 << 3)
#define DAC_45_DIFF_EN   (1 << 2)
#define DAC_23_DIFF_EN   (1 << 1)
#define DAC_01_DIFF_EN   (1 << 0)

//BRD CONFIG params
#define DAC15_BRDCAST_EN  (1 << 15)
#define DAC14_BRDCAST_EN  (1 << 14)
#define DAC13_BRDCAST_EN  (1 << 13)
#define DAC12_BRDCAST_EN  (1 << 12)
#define DAC11_BRDCAST_EN  (1 << 11)
#define DAC10_BRDCAST_EN  (1 << 10)
#define DAC9_BRDCAST_EN   (1 <<  9)
#define DAC8_BRDCAST_EN   (1 <<  8)
#define DAC7_BRDCAST_EN   (1 <<  7)
#define DAC6_BRDCAST_EN   (1 <<  6)
#define DAC5_BRDCAST_EN   (1 <<  5)
#define DAC4_BRDCAST_EN   (1 <<  4)
#define DAC3_BRDCAST_EN   (1 <<  3)
#define DAC2_BRDCAST_EN   (1 <<  2)
#define DAC1_BRDCAST_EN   (1 <<  1)
#define DAC0_BRDCAST_EN   (1 <<  0)

//SYN CONFIG params
/*
When set to 1 the corresponding DAC output is set to update in
response to an LDAC trigger (synchronous mode).
When cleared to 0 the corresponding DAC output is set to update
immediately (asynchronous mode).
*/
#define DAC15_SYNC_EN (1 << 15)
#define DAC14_SYNC_EN (1 << 14)
#define DAC13_SYNC_EN (1 << 13)
#define DAC12_SYNC_EN (1 << 12)
#define DAC11_SYNC_EN (1 << 11)
#define DAC10_SYNC_EN (1 << 10)
#define DAC9_SYNC_EN  (1 <<  9)
#define DAC8_SYNC_EN  (1 <<  8)
#define DAC7_SYNC_EN  (1 <<  7)
#define DAC6_SYNC_EN  (1 <<  6)
#define DAC5_SYNC_EN  (1 <<  5)
#define DAC4_SYNC_EN  (1 <<  4)
#define DAC3_SYNC_EN  (1 <<  3)
#define DAC2_SYNC_EN  (1 <<  2)
#define DAC1_SYNC_EN  (1 <<  1)
#define DAC0_SYNC_EN  (1 <<  0)

//TOGGCONFIG0
/*
Enables toggle mode operation and configures the toggle pin or soft
toggle bit:
00 = Toggle mode disabled
01 = Toggle mode enabled: TOGGLE0
10 = Toggle mode enabled: TOGGLE1
11 = Toggle mode enabled: TOGGLE2
*/
#define DAC15_TOGG_SHIFT 7
#define DAC14_TOGG_SHIFT 6
#define DAC13_TOGG_SHIFT 5
#define DAC12_TOGG_SHIFT 4
#define DAC11_TOGG_SHIFT 3
#define DAC10_TOGG_SHIFT 2
#define DAC9_TOGG_SHIFT  1
#define DAC8_TOGG_SHIFT  0
#define DAC7_TOGG_SHIFT  7
#define DAC6_TOGG_SHIFT  6
#define DAC5_TOGG_SHIFT  5
#define DAC4_TOGG_SHIFT  4
#define DAC3_TOGG_SHIFT  3
#define DAC2_TOGG_SHIFT  2
#define DAC1_TOGG_SHIFT  1
#define DAC0_TOGG_SHIFT  0

#define TOGGLE_DISABLE 0x00
#define TOGGLE_0       0x01
#define TOGGLE_1       0x10
#define TOGGLE_2       0x11

#define DAC15_AB_TOGG_EN_BITS  (1 << 15) | (1 << 14)
#define DAC14_AB_TOGG_EN_BITS  (1 << 13) | (1 << 12)
#define DAC13_AB_TOGG_EN_BITS  (1 << 11) | (1 << 10)
#define DAC12_AB_TOGG_EN_BITS  (1 <<  9) | (1 <<  8)
#define DAC11_AB_TOGG_EN_BITS  (1 <<  7) | (1 <<  6)
#define DAC10_AB_TOGG_EN_BITS  (1 <<  5) | (1 <<  4)
#define DAC9_AB_TOGG_EN_BITS  (1 <<  3) | (1 <<  2)
#define DAC8_AB_TOGG_EN_BITS  (1 <<  1) | (1 <<  0)

//TOGGCONFIG1
#define DAC7_AB_TOGG_EN_BITS (1 << 15) | (1 << 14)
#define DAC6_AB_TOGG_EN_BITS (1 << 13) | (1 << 12)
#define DAC5_AB_TOGG_EN_BITS (1 << 11) | (1 << 10)
#define DAC4_AB_TOGG_EN_BITS (1 <<  9) | (1 <<  8)
#define DAC3_AB_TOGG_EN_BITS (1 <<  7) | (1 <<  6)
#define DAC2_AB_TOGG_EN_BITS (1 <<  5) | (1 <<  4)
#define DAC1_AB_TOGG_EN_BITS (1 <<  3) | (1 <<  2)
#define DAC0_AB_TOGG_EN_BITS (1 <<  1) | (1 <<  0)

//DACPWDWN
/*
When set to 1 the corresponding DAC is in power-down mode and
its output is connected to GND through a 10-kOhm internal resistor.
*/
#define DAC15_EN ~(1 << 15)
#define DAC14_EN ~(1 << 14)
#define DAC13_EN ~(1 << 13)
#define DAC12_EN ~(1 << 12)
#define DAC11_EN ~(1 << 11)
#define DAC10_EN ~(1 << 10)
#define DAC9_EN  ~(1 <<  9)
#define DAC8_EN  ~(1 <<  8)
#define DAC7_EN  ~(1 <<  7)
#define DAC6_EN  ~(1 <<  6)
#define DAC5_EN  ~(1 <<  5)
#define DAC4_EN  ~(1 <<  4)
#define DAC3_EN  ~(1 <<  3)
#define DAC2_EN  ~(1 <<  2)
#define DAC1_EN  ~(1 <<  1)
#define DAC0_EN  ~(1 <<  0)

//DACRANGEn
/*
All others: invalid
The two outputs of a differential DAC pair must be configured to the
same output range prior to setting them up as a differential pair.
a: 15, 11, 7 or 3
b: 14, 10, 6 or 2
c: 13, 9, 5 or 1
d: 12, 8, 4 or 0
*/
typedef enum dac_range{
    DAC_RANGE_0_5   = 0x00,
    DAC_RANGE_0_10  = 0x01,
    DAC_RANGE_0_20  = 0x02,
    DAC_RANGE_0_40  = 0x04,
    DAC_RANGE_5_5   = 0x09,
    DAC_RANGE_10_10 = 0x0a,
    DAC_RANGE_20_5  = 0x0c,
    DAC_RANGE_2_2   = 0x0e,
}dac_range_e;
#define DACA_RANGE (1 << 15)
/*
15-12 DACa-RANGE[3:0] W 0h Sets the output range for the corresponding DAC.
11-8 DACb-RANGE[3:0] W 0h
7-4 DACc-RANGE[3:0] W 0h
3-0 DACd-RANGE[3:0] W 0h
*/



//TRIGGER
/*
If soft toggle is enabled set, this bit controls the toggle between
values for those DACs that have been set in toggle mode 2 in the
TOGGCONFIG register. Set to 1 to update to Register B and clear to
0 for Register A.
*/
//Set this bit to 1 to clear an alarm event. Not applicable for a DACBUSY alarm event.
#define ALM_RESET (1 << 8)
#define AB_TOG2 (1 << 7)
#define AB_TOG1 (1 << 6)
#define AB_TOG0 (1 << 5)

/*Set this bit to 1 to synchronously load those DACs who have been
set in synchronous mode in the SYNCCONFIG register.*/
#define LDAC    (1 << 4)
// When set to the reserved code 1010 resets the device to its default state.
#define SOFT_RESET 0b1010

#endif
